/*
 * RAS Controler  - RasCon -
 *
 *          Copyright (C) 1998  Shuichi Kitaguchi <kit@Mew.org>
 *
 * Created: Apr 6, 1998
 * Time-stamp: <1999/08/30 20:24:10 kit>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either versions 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with RasCon, see the file COPYING.  If not, write to the Free
 * Software Foundation Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */

#include <windows.h>
#include <conio.h>
#include <ras.h>
#include <raserror.h>
#include <io.h>
#include <stdio.h>
#ifdef ENABLE_DIALKEEPER
#include "dklib.h"
#endif


#define RASCON_VERSION   "0.3.1"

#define DIAL_TIMEOUT     180000	/* timeout for dialing (msec) */


#define ENTRY_ALL        0xffff

#define STATUS_NONE          0
#define STATUS_ERRORWHENDIAL 1
#define STATUS_RETRY         2

int           iStatus        = STATUS_NONE;

BOOL          fVerbose       = FALSE;
BOOL          fInputUserName = FALSE;
BOOL          fInputPassword = FALSE;
BOOL          fStoreUP       = FALSE;
BOOL          fRedial        = FALSE;
#ifdef ENABLE_DIALKEEPER
BOOL          fDialKeeper    = FALSE;
#endif

DWORD         dwRedialInterval;
DWORD         dwRedialCount;
DWORD         dwRetryCount;
DWORD         dwEntryNum;
RASENTRYNAME  ren[RAS_MaxEntryName];
DWORD         dwConnectionNum;
RASCONN       rcon[RAS_MaxEntryName];
HANDLE        hConnection;

HRASCONN      hCurRasConn;
RASCONNSTATUS rcStatus;
RASCONNSTATE  rcsPrevRasConnState = 10000; /* xxx */
#ifdef ENABLE_DIALKEEPER
DKCOM         dkcom;
#endif



#ifdef ENABLE_DIALKEEPER
BOOL WINAPI ConsoleHandler_DK( DWORD dwCtrlType )
{
  DK_TermClient( &dkcom );
  iStatus = STATUS_ERRORWHENDIAL;
  
  return ( TRUE );
}
#endif


int m_getc( void )
{
  if ( _isatty( _fileno( stdin ) ) ){
    return ( _getch() );
  } else {
    return ( getchar() );
  }
}


BOOL WINAPI ConsoleHandler( DWORD dwCtrlType )
{
  /* input Ctrl-c code */

  printf("Abort.\n");
  fflush(stdout);
  iStatus = STATUS_ERRORWHENDIAL;
  if ( hCurRasConn )
    RasHangUp( hCurRasConn );
  SetEvent( hConnection );
  
  return ( TRUE );
}


BOOL ReadRasConnections( VOID )
{
  DWORD dwRet;
  DWORD dwSize;

  dwSize         = sizeof(rcon);
  rcon[0].dwSize = sizeof(RASCONN);
  dwRet = RasEnumConnections( rcon, &dwSize, &dwConnectionNum );

  if ( dwRet != 0 ) return ( FALSE );

  return ( TRUE );
}


BOOL ReadRasEntries( VOID )
{
  DWORD dwSize;
  DWORD dwRet;

  dwSize = sizeof(ren);
  ren[0].dwSize = sizeof(RASENTRYNAME);
  dwRet = RasEnumEntries( NULL, NULL, ren, &dwSize, &dwEntryNum );

  if ( dwRet == 0 ){
    return ( TRUE );
  } else {
    printf("Error: ReadRasEntries: [%ld](%ld) happens.\n",dwRet,GetLastError());
    return ( FALSE );
  }
}


VOID PrintRasStatusMessage( RASCONNSTATE rcs )
{
  LPSTR         lpMsg;

  switch ( rcs ){
  case RASCS_OpenPort:
    lpMsg = "Opening port...";
    break;
  case RASCS_PortOpened:
    lpMsg = "Port has been opened.";
    break;
  case RASCS_ConnectDevice:
    lpMsg = "Connecting device...";
    break;
  case RASCS_DeviceConnected:
    lpMsg = "Device has connected.";
    break;
  case RASCS_AllDevicesConnected:
    lpMsg = "All devices have connected.";
    break;
  case RASCS_Authenticate:
    lpMsg = "Authentication is starting...";
    break;
  case RASCS_AuthNotify:
#if 0
    lpMsg = "Authentication event has occured.";
#else
    lpMsg = "";
#endif
    break;
  case RASCS_AuthRetry:
    lpMsg = "Retrying authentication...";
    break;
  case RASCS_AuthCallback:
    lpMsg = "Requesting callback number...";
    break;
  case RASCS_AuthChangePassword:
    lpMsg = "Requesting to change password...";
    break;
  case RASCS_AuthProject:
    lpMsg = "Projection is starting...";
    break;
  case RASCS_AuthLinkSpeed:
    lpMsg = "Link-speed calculation is starting...";
    break;
  case RASCS_AuthAck:
    lpMsg = "Authentication is being acknowledged...";
    break;
  case RASCS_ReAuthenticate:
    lpMsg = "ReAuthenticatication is starting...";
    break;
  case RASCS_Authenticated:
    lpMsg = "Authentication completed.";
    break;
  case RASCS_PrepareForCallback:
    lpMsg = "Preparing for callback...";
    break;
  case RASCS_WaitForModemReset:
    lpMsg = "Waiting for moodem reset...";
    break;
  case RASCS_WaitForCallback:
    lpMsg = "Waiting for callback...";
    break;
  case RASCS_Projected:
    lpMsg = "Projection completed.";
    break;

  case RASCS_StartAuthentication:
    lpMsg = "Authentication is being initiated.";
    break;
  case RASCS_CallbackComplete:
    lpMsg = "Callback completed.";
    break;
  case RASCS_LogonNetwork:
    lpMsg = "Logging on network...";
    break;

  case RASCS_Connected:
    lpMsg = "Connected.";
    SetEvent( hConnection );
    break;
  case RASCS_Disconnected:
    lpMsg = "Disconnected.";
    RasHangUp( hCurRasConn );
    SetEvent( hConnection );
    break;

  default:
    lpMsg = "Unknown RASCONNSTATE.";
    break;
  }
  if ( fVerbose ){
    if ( strcmp(lpMsg,"") ) printf("%s(%d)\n",lpMsg,rcs);
  } else {
    if ( strcmp(lpMsg,"") ) printf("%s\n",lpMsg);
  }
  fflush(stdout);
}


VOID WINAPI PrintRasStatus( UINT         uiMsg,
			    RASCONNSTATE rcs,
			    DWORD        dwError )
{
  if ( dwError != 0 ){
    if ( dwError == ERROR_LINE_BUSY ){
      if ( fRedial ){
	iStatus = STATUS_RETRY;
      } else {
	printf("Error: Line is BUSY.");
	fflush(stdout);
	iStatus = STATUS_ERRORWHENDIAL;
      }	/* if ( fRedial ){ */
    } else {
      printf("Error: PrintRasStatus: [%ld](%ld) happens.\n",
	     dwError,GetLastError());
      fflush(stdout);
      iStatus = STATUS_ERRORWHENDIAL;
    } /* if ( dwError == ERROR_LINE_BUSY ){ */
    rcsPrevRasConnState = 10000;
    RasHangUp( hCurRasConn );
    SetEvent( hConnection );
    return;
  } /* if ( dwError != 0 ){ */

  if ( rcsPrevRasConnState != rcs ){
    PrintRasStatusMessage( rcs );
    rcsPrevRasConnState = rcs;
  }
}


BOOL ReadLine( LPSTR lpLine, int mode )
{
  int  i=0;
  char c,st[2];
  BOOL fRet=FALSE;

  st[1] = 0;
  while ( i < (PWLEN+1) ){
    c = m_getc();
    if ( (c==0x0d) || (c==0x0a) ){ /* return */
      *lpLine = 0x00;
      fRet = TRUE;
      break;
    }
    if ( c == 0x03 ){		/* \C-c */
      fRet = FALSE;
      break;
    }
    if ( mode != 0 ){
      st[0] = c;
      printf("%s",st);
    }
    *lpLine++ = c;
    i++;
  } /* while ( i < (PWLEN+1) ){ */

  printf("\n");

  return ( fRet );
}


BOOL DialUpRas( DWORD dwSelectEntryNum )
{
  RASDIALPARAMS rdp,rdp2;
  BOOL          fPassword;
  DWORD         dwRet;

  ReadRasEntries();

  if ( dwSelectEntryNum >= dwEntryNum ){
    printf("Error: <num> is illegal.\n");
    return ( FALSE );
  }

  memset(&rdp,0,sizeof(rdp));
  rdp.dwSize = sizeof(rdp);
  strcpy(rdp.szEntryName,ren[dwSelectEntryNum].szEntryName);
  dwRet = RasGetEntryDialParams(NULL,&rdp,&fPassword);
  if ( dwRet != 0 ){
    printf("Error: RasGetEntryDialParams: [%ld](%ld) happens.\n",dwRet,GetLastError());
    return ( FALSE );
  }

  /* backup rdp */
  rdp2 = rdp;

  /* no user name and read it */
  if ( fInputUserName || (rdp.szUserName[0]==0) ){
    printf("Enter User name : ");
    fflush(stdout);
    if ( ! ReadLine( rdp.szUserName, 1 ) ){
      printf("Error: Abort.\n");
      return ( FALSE );
    }
  } /* if ( rdp.szUserName[0] == 0 ){ */

  /* no password and read it */
  if ( fInputPassword || (rdp.szPassword[0]==0) ){
    printf("Enter Password for (%s): ",rdp.szUserName);
    fflush(stdout);
    if ( ! ReadLine( rdp.szPassword, 0 ) ){
      printf("Error: Abort.\n");
      return ( FALSE );
    }
  } /* if ( rdp.szPassword[0] == 0 ){ */

  if ( fVerbose ){
    printf("Phone No - [%s] / Callback No [%s] / User - [%s]\n",
	   rdp.szPhoneNumber,rdp.szCallbackNumber,rdp.szUserName);
    printf("Domain - [%s]\n",rdp.szDomain);
  }

  printf("Dialing [%s]...\n",ren[dwSelectEntryNum].szEntryName);

#ifdef ENABLE_DIALKEEPER
  if ( fDialKeeper ){
    /* set handler */
    SetConsoleCtrlHandler( ConsoleHandler_DK, TRUE );

    /* DialKeeper mode */
    if ( ! DK_InitClient( &dkcom ) ){
      printf("Error: Fail to initialize communication library.\n");
      return ( FALSE );
    } /* if ( DK_InitClient( &dkcom ) ){ */
    
    dkcom.com  = DKCOM_COM_DIALUP;
    dkcom.num  = (BYTE)dwSelectEntryNum;
    dkcom.cnum = (BYTE)dwRedialCount;
    dkcom.tnum = (BYTE)dwRedialInterval;
    strcpy( dkcom.user, rdp.szUserName );
    strcpy( dkcom.pass, rdp.szPassword );

    if ( ! DK_WriteSData( &dkcom, DKSDATA_STATUS_CREATED ) )
      if ( fVerbose )
	printf("Error: DialUpRas: DK_WriteSData fails.\n");

    if ( ! DK_SendMessage() ){
      printf("Error: DialKeeper is not found.\n");
      if ( ! DK_TermClient( &dkcom ) )
	if ( fVerbose )
	  printf("Error: DialUpRas: DK_TermClient fails.\n");
      return ( FALSE );
    } else {
      if ( fVerbose )
	printf("Error: DialUpRas: DK_SendMessage fails.\n");
    }

    if ( DK_WaitSignal( &dkcom ) )
      if ( ! DK_ReadSData( &dkcom, DKSDATA_STATUS_FINISHED ) )
	if ( fVerbose )
	  printf("Error: DialUpRas: DK_ReadSData fails.\n");
    else
      if ( fVerbose )
	printf("Error: DialUpRas: DK_WaitSignal fails.\n");

    if ( ! DK_TermClient( &dkcom ) )
      if ( fVerbose )
	printf("Error: DialUpRas: DK_TermClient fails.\n");

    /* remove handler */
    SetConsoleCtrlHandler( ConsoleHandler_DK, FALSE );

    if ( dkcom.ret != DKCOM_RET_SUCCESS ){
      printf("Error: Dialing-up fails.\n");
      return ( FALSE );
    }
  } else {
#endif
    /* set handler */
    SetConsoleCtrlHandler( ConsoleHandler, TRUE );
    
    /* dialing */
    hCurRasConn=NULL;
    dwRetryCount=0;
    while ( 1 ){
      dwRet = RasDial(NULL,NULL,&rdp,0,(RASDIALFUNC)PrintRasStatus,&hCurRasConn);
      if ( dwRet != 0 ){
	printf("Error: RasDial: [%ld](%ld) happens.\n",dwRet,GetLastError());
	fflush(stdout);
	return ( FALSE );
      } /* if ( dwRet == 0 ){ */

      dwRet = WaitForSingleObject( hConnection, DIAL_TIMEOUT );
      if ( dwRet == WAIT_FAILED ){
	printf("Error: WaitForSingleObject: [%ld](%ld) happens.\n",
	     dwRet,GetLastError());
	fflush(stdout);
	SetConsoleCtrlHandler( ConsoleHandler, FALSE );
	return ( FALSE );
      } else {
	/* retrying */
	if ( iStatus == STATUS_RETRY ){
	  dwRetryCount++;
	  if ( dwRetryCount >= dwRedialCount ){
	    /* retry over */
	    printf("Error: Line has been BUSY.\n");
	    fflush(stdout);
	    iStatus = STATUS_ERRORWHENDIAL;
	  } else {
	    iStatus = STATUS_NONE;
	    ResetEvent( hConnection );
	    printf("Warning: Line is BUSY. Wait interval time and retry...\n");
	    fflush(stdout);
	    Sleep(dwRedialInterval*1000);
	    continue;
	  }
	} /* if ( iStatus == STATUS_RETRY ){ */
	/* error happens */
	if ( iStatus == STATUS_ERRORWHENDIAL ){
	  SetConsoleCtrlHandler( ConsoleHandler, FALSE );
	  return ( FALSE );
	}
	/* successfully connected */
	break;
      }
    } /* while ( 1 ){ */
    
    /* remove handler */
    SetConsoleCtrlHandler( ConsoleHandler, FALSE );
#ifdef ENABLE_DIALKEEPER
  } /* if ( fDialKeeper ){ */
#endif

  /* DO NOT store user name and password */
  if ( !fStoreUP ){
    strcpy( rdp2.szUserName, rdp.szUserName ); /* store user name */
    dwRet = RasSetEntryDialParams( NULL, &rdp2, FALSE ); /* set old parameter */
    if ( dwRet != 0 ){
      printf("Error: RasSetEntryDialParams: [%ld](%ld) happens.\n",
	     dwRet,GetLastError());
      return ( FALSE );
    }
  }

  printf("Success: Entry [%s] has been connected.\n",
	 ren[dwSelectEntryNum].szEntryName);
  fflush(stdout);

  return ( TRUE );
}


BOOL HangUpRas( DWORD dwSelectEntryNum )
{
  DWORD i;

  if ( ! ReadRasEntries() ) return ( FALSE );
  if ( ! ReadRasConnections() ) return ( FALSE );

  if ( (dwSelectEntryNum!=ENTRY_ALL) && (dwSelectEntryNum>=dwEntryNum) ){
    printf("Error: <num> is illegal.\n");
    return ( FALSE );
  }

#ifdef ENABLE_DIALKEEPER
  if ( fDialKeeper ){
    /* DialKeeper mode */
    DKCOM dkcom;
    if ( ! DK_InitClient( &dkcom ) ){
      printf("Error: Fail to initialize communication library.\n");
      return ( FALSE );
    } /* if ( DK_InitClient( &dkcom ) ){ */
    
    dkcom.com = DKCOM_COM_HANGUP;
    dkcom.num = (BYTE)dwSelectEntryNum;

    if ( ! DK_WriteSData( &dkcom, DKSDATA_STATUS_CREATED ) )
      if ( fVerbose )
	printf("Error: HangUpRas: DK_WriteSData fails.\n");

    if ( ! DK_SendMessage() ){
      printf("Error: DialKeeper is not found.\n");
      DK_TermClient( &dkcom );
      return ( FALSE );
    } else {
      if ( fVerbose )
	printf("Error: HangUpRas: DK_SendMessage fails.\n");
    }
    if ( ! DK_WaitSignal( &dkcom ) )
      if ( fVerbose )
	printf("Error: HangUpRas: DK_WaitSignal fails.\n");
    
    if ( ! DK_ReadSData( &dkcom, DKSDATA_STATUS_FINISHED ) )
      if ( fVerbose )
	printf("Error: HangUpRas: DK_ReadSData fails.\n");

    if ( ! DK_TermClient( &dkcom ) )
      if ( fVerbose )
	printf("Error: HangUpRas: DK_TermClient fails.\n");

    if ( dkcom.ret != DKCOM_RET_SUCCESS ){
      printf("Error: Hanging-up fails.\n");
      return ( FALSE );
    }
  } else {
#endif
    for ( i=0; i<dwConnectionNum; i++ ){
      if ( (dwSelectEntryNum==ENTRY_ALL) ||
	   ! strcmp( ren[dwSelectEntryNum].szEntryName, rcon[i].szEntryName ) ){
	RasHangUp( rcon[i].hrasconn );
	break;
      }
    } /* for ( i=0; i<dwConnectionNum; i++ ){ */
    if ( i >= dwConnectionNum ){
      if ( dwSelectEntryNum == ENTRY_ALL ){
	printf("Success: All entries have been disconnected.\n");
	return ( TRUE );
      } else {
	printf("Error: Entry [%s] does not connected.\n",
	       ren[dwSelectEntryNum].szEntryName);
	return ( FALSE );
      }
    } /* if ( i >= dwConnectionNum ){ */
#ifdef ENABLE_DIALKEEPER
  }
#endif
  if ( dwSelectEntryNum == ENTRY_ALL ){
    printf("Success: All entries have been disconnected.\n");
  } else {
    printf("Success: Entry [%s] has been disconnected.\n",
	   ren[dwSelectEntryNum].szEntryName);
  }
  fflush(stdout);
  return ( TRUE );
}


VOID ListRas( VOID )
{
  DWORD i,j;
  BOOL  fConnect;

  ReadRasEntries();
  ReadRasConnections();

  if ( dwEntryNum == 0 ){
    printf("No entries.\n");
  } else {
    for ( i=0; i<dwEntryNum; i++ ){
      fConnect=FALSE;
      for ( j=0; j<dwConnectionNum; j++ ){
	if ( ! strcmp(ren[i].szEntryName,rcon[j].szEntryName) ){
	  fConnect=TRUE;
	  break;
	}
      } /* for ( j=0; j<dwConnectionNum; j++ ){ */
      if ( fConnect ){
	printf("%d : connect    : %s\n",i,ren[i].szEntryName);
      } else {
	printf("%d : disconnect : %s\n",i,ren[i].szEntryName);
      } /* if ( fConnect ){ */
      fflush(stdout);
    } /* for ( i=0; i<dwEntryNum; i++ ){ */
  }
}


BOOL RemovePassword( DWORD dwSelectEntryNum )
{
  RASDIALPARAMS rdp;
  BOOL          fPassword;
  DWORD         dwRet;

  ReadRasEntries();

  if ( dwSelectEntryNum >= dwEntryNum ){
    printf("Error: <num> is illegal.\n");
    return ( FALSE );
  }

  memset(&rdp,0,sizeof(rdp));
  rdp.dwSize = sizeof(rdp);
  strcpy(rdp.szEntryName,ren[dwSelectEntryNum].szEntryName);
  dwRet = RasGetEntryDialParams(NULL,&rdp,&fPassword);
  if ( dwRet != 0 ){
    printf("Error: RasGetEntryDialParams: [%ld](%ld) happens.\n",dwRet,GetLastError());
    return ( FALSE );
  }

  dwRet = RasSetEntryDialParams( NULL, &rdp, TRUE ); /* remove password */
  if ( dwRet != 0 ){
    printf("Error: RasSetEntryDialParams: [%ld](%ld) happens.\n",
	   dwRet,GetLastError());
    return ( FALSE );
  }
  
  printf("Success: Entry [%s]'s Password has been removed.\n",
	 ren[dwSelectEntryNum].szEntryName);

  return ( TRUE );
}


VOID PrintUsage( VOID )
{
  printf("RAS Controler  - RasCon -   Version %s",RASCON_VERSION);
#ifdef ENABLE_DIALKEEPER
  printf("+DK");
#endif
  printf("\n");
  printf("          Copyright (C) 1998  Shuichi Kitaguchi\n");
  printf("\n");
  printf("Usage  : RasCon <options>\n");
  printf("Option :  -c <num>          connect(dial-up).\n");
  printf("          -d [num]          disconnect(hang-up).\n");
#ifdef ENABLE_DIALKEEPER
  printf("          -k                DialKeeper mode.\n");
#endif
  printf("          -l                list nums/entries in RAS Phonebook.\n");
  printf("          -u                input UserName.\n");
  printf("          -p                input Password.\n");
  printf("          -s                store Password in Phonebook.\n");
  printf("          -r <num>          remove Password in Phonebook.\n");
  printf("          -i <cnum> <tnum>  redial count and interval time(sec).\n");
  printf("          -v                verbose mode.\n");
  printf("          -h                show this message.\n");
}


void main( int argc, char *argv[] )
{
  int   i;
  DWORD dwSelectEntryNum;
  BOOL  fList           = FALSE;
  BOOL  fConnection     = FALSE;
  BOOL  fDisconnection  = FALSE;
  BOOL  fUsage          = FALSE;
  BOOL  fRemovePassword = FALSE;

  for ( i=1; i<argc; i++ ){
    if ( ! strcmp(argv[i],"-l") ){
      fList = TRUE;
    } else if ( ! strcmp(argv[i],"-c") ){
      fConnection = TRUE;
      i++;
      if ( i >= argc ){
	printf("Error: <num> is not specified with \"-c\".\n");
	fUsage=TRUE;
	break;
      } else {
	dwSelectEntryNum = atol(argv[i]);
	if ( (dwSelectEntryNum==0) && (argv[i][0]!='0') ){
	  printf("Error: <num> is illegal.\n");
	  fUsage=TRUE;
	  break;
	}
      }	/* if ( i >= argc ){ */
    } else if ( ! strcmp(argv[i],"-d") ){
      fDisconnection = TRUE;
      i++;
      if ( i >= argc ){
	i--;
	dwSelectEntryNum = ENTRY_ALL;
      } else {
	dwSelectEntryNum = atol(argv[i]);
	if ( (dwSelectEntryNum==0) && (argv[i][0]!='0') ){
	  i--;
	  dwSelectEntryNum = ENTRY_ALL;
	}
      }	/* if ( i >= argc ){ */
    } else if ( ! strcmp(argv[i],"-r") ){
      fRemovePassword = TRUE;
      i++;
      if ( i >= argc ){
	printf("Error: <num> is not specified with \"-r\".\n");
	fUsage=TRUE;
	break;
      } else {
	dwSelectEntryNum = atol(argv[i]);
	if ( (dwSelectEntryNum==0) && (argv[i][0]!='0') ){
	  printf("Error: <num> is illegal.\n");
	  fUsage=TRUE;
	  break;
	}
      }	/* if ( i >= argc ){ */
    } else if ( ! strcmp(argv[i],"-i") ){
      fRedial = TRUE;
      i++;
      if ( i >= argc ){
	printf("Error: <cnum> is not specified with \"-i\".\n");
	fUsage=TRUE;
	break;
      } else {
	dwRedialCount = atol(argv[i]);
	if ( dwRedialCount <= 0 ){
	  printf("Error: <cnum> is illegal.\n");
	  fUsage=TRUE;
	  break;
	}
      }	/* if ( i >= argc ){ */
      i++;
      if ( i >= argc ){
	printf("Error: <tnum> is not specified with \"-i\".\n");
	fUsage=TRUE;
	break;
      } else {
	dwRedialInterval = atol(argv[i]);
	if ( dwRedialInterval == 0 ){
	  printf("Error: <tnum> is illegal.\n");
	  fUsage=TRUE;
	  break;
	}
      }	/* if ( i >= argc ){ */
#ifdef ENABLE_DIALKEEPER
    } else if ( ! strcmp(argv[i],"-k") ){
      fDialKeeper = TRUE;
#endif
    } else if ( ! strcmp(argv[i],"-h") ){
      fUsage = TRUE;
    } else if ( ! strcmp(argv[i],"-v") ){
      fVerbose = TRUE;
    } else if ( ! strcmp(argv[i],"-u") ){
      fInputUserName = TRUE;
    } else if ( ! strcmp(argv[i],"-p") ){
      fInputPassword = TRUE;
    } else if ( ! strcmp(argv[i],"-s") ){
      fStoreUP = TRUE;
    }
  } /* for ( i=1; i<argc; i++ ){ */

  if ( fUsage ){
    PrintUsage();
  } else if ( fList ){
    ListRas();
  } else if ( fRemovePassword ){
    RemovePassword( dwSelectEntryNum );
  } else if ( fConnection ){
    hConnection = CreateEvent( NULL, TRUE, FALSE, NULL );
    if ( hConnection == NULL ){
      printf("Error: cannot create event object.\n");
      exit ( -1 );
    }
    DialUpRas( dwSelectEntryNum );
  } else if ( fDisconnection ){
    HangUpRas( dwSelectEntryNum );
  } else {
    PrintUsage();
  }
}
